import scanpy as sc
from anndata import AnnData

from ._cellbin_plot import plot_cellbin_discrete, plot_cellbin_gradient, plot_zoom_cellbin
from ._nhood_heatmap import nhood_heatmap
from ._utils import getDefaultColors



import numpy as np
from anndata import AnnData


def removeBiasGenes(adata: AnnData) -> AnnData:
    """Remove unused genes for human singcle cell analysis

    Args:
        adata (AnnData): Anndata object

    Returns
    -------
        AnnData: filtered anndata
    """
    malat1 = adata.var_names.str.startswith("MALAT1")
    MTgenes = adata.var_names.str.startswith("MT-")
    hb_genes = adata.var_names.str.contains("^HB[^(P)]")
    RPgenes = adata.var_names.str.startswith("RP") & adata.var_names.str.contains("-")
    RPgenes2 = adata.var_names.str.contains("^RP[SL]")
    CTCgenes = adata.var_names.str.startswith("CTC") & adata.var_names.str.contains("-")
    MIRgenes = adata.var_names.str.startswith("MIR")
    ACgenes = adata.var_names.str.contains("^AC[0-9]") & adata.var_names.str.contains(".")
    CTgenes = adata.var_names.str.startswith("CT") & adata.var_names.str.contains("-")
    LINCgenes = adata.var_names.str.contains("^LINC[0-9]")
    ALgenes = adata.var_names.str.contains("^AL") & adata.var_names.str.contains(".")
    remove_genes = (
        malat1 | MTgenes | hb_genes | RPgenes | RPgenes2 | CTCgenes | MIRgenes | ACgenes | CTgenes | LINCgenes | ALgenes
    )
    keep = np.invert(remove_genes)
    adata = adata[:, keep]



import pandas as pd
import squidpy as sq
from anndata import AnnData


def nhood_enrichment(
    adata: AnnData,
    coord_type: str = "generic",
    library_key: str = "batch",
    radius: float = 30,
    cluster_key: str = "region",
) -> pd.DataFrame:
    """_summary_

    Args:
        adata (AnnData): spatial anndata
        coord_type (str, optional): Type of coordinate system., defaults
            to "generic"
        library_key (str, optional): batch info, defaults to "batch"
        radius (float, optional): Compute the graph based on
            neighborhood radius, defaults to 30
        cluster_key (str, optional): region or cell cluster key,
            defaults to "region"

    Returns
    -------
        pd.DataFrame: a dataframe of neighborhood enrichment
    """
    sq.gr.spatial_neighbors(adata, coord_type=coord_type, library_key=library_key, radius=radius)
    sq.gr.nhood_enrichment(adata, cluster_key=cluster_key)
    region_number = adata.obs[cluster_key].value_counts()[adata.obs[cluster_key].cat.categories]
    nhood_counts = pd.DataFrame(
        adata.uns[f"{cluster_key}_nhood_enrichment"]["count"],
        index=adata.obs[cluster_key].cat.categories,
        columns=adata.obs[cluster_key].cat.categories,
    )
    nhood_percents = nhood_counts / region_number
    return nhood_percents



from collections import OrderedDict

import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import seaborn as sns
from anndata import AnnData
from matplotlib.gridspec import GridSpec


def getDEG(
    adata: AnnData,
    cluster: str,
    qval_cutoff: float = 0.1,
    mean_expr_cutoff: float = 0.2,
    top_genes: int = 200,
    layer: str = None,
    save: str = "volcano_plot.pdf",
    key_added: str = "rank_genes_groups",
) -> dict:
    """Get DEGs for rank_genes_groups

    Args:
        adata (AnnData): adata object
        cluster (str): cluster tag
        qval_cutoff (float, optional): qvalue cutoff, defaults to 0.1
        mean_expr_cutoff (float, optional): mean expression cutoff,
            defaults to 0.2
        top_genes (int, optional): number of top genes, defaults to 200
        layer (str, optional): adata layers, defaults to None
        save (str, optional): if or not save volcano plot, defaults to
            "volcano_plot.pdf"
        key_added (str, optional): DEG key, defaults to
            "rank_genes_groups"

    Returns
    -------
        dict: a DEG dict.
    """
    # sc.tl.rank_genes_groups(adata, cluster, method='wilcoxon')
    result = adata.uns[key_added]
    groups = result["names"].dtype.names
    DEG_dict = OrderedDict()
    diff_genes = pd.DataFrame(
        columns=[
            "group",
            "names",
            "logfoldchanges",
            "pvals",
            "pvals_adj",
            "mean_expression",
        ]
    )
    for group in groups:
        df = pd.DataFrame({key: result[key][group] for key in ["names", "logfoldchanges", "pvals", "pvals_adj"]})
        df["group"] = group
        cluster_cells = adata.obs[cluster] == group
        cluster_data = adata[cluster_cells, :]
        if layer:
            mean_expression = np.mean(cluster_data.layers[layer].toarray(), axis=0)
        else:
            mean_expression = np.mean(cluster_data.X.toarray(), axis=0)
        df = pd.merge(
            df,
            pd.Series(mean_expression, index=cluster_data.var_names, name="mean_expression"),
            left_on="names",
            right_index=True,
        )
        # diff_genes = diff_genes.append(df[['group', 'names', 'logfoldchanges', 'pvals','pvals_adj','mean_expression']])
        diff_genes = pd.concat(
            [
                diff_genes,
                df[
                    [
                        "group",
                        "names",
                        "logfoldchanges",
                        "pvals",
                        "pvals_adj",
                        "mean_expression",
                    ]
                ],
            ],
            axis=0,
        )
        df_filtered = df[
            (df["pvals_adj"] < qval_cutoff) & (df["mean_expression"] > mean_expr_cutoff) & (df["logfoldchanges"] > 0)
        ]
        df_filtered.set_index("names", inplace=True, drop=False)
        # sort rows based on logfoldchange
        df_filtered = df_filtered.iloc[df_filtered["logfoldchanges"].argsort()[::-1]]
        DEG_dict[group] = df_filtered.iloc[0:top_genes, :].index.tolist()
    # 绘制火山图
    ncol = 4
    nrow = len(groups) // 4 + 1
    fig = plt.figure(figsize=(ncol * 6, nrow * 6))
    gs = GridSpec(nrow, ncol, figure=fig)  # 将整个画布分成len(groups)列
    for i, group in enumerate(groups):
        rownum = i // 4
        colnum = i - ((i // 4) * 4)
        ax = fig.add_subplot(gs[rownum, colnum])
        data = diff_genes[diff_genes["group"] == group]
        sns.scatterplot(data=data, x="logfoldchanges", y="mean_expression", color="black")
        sig_genes = data.loc[(data["pvals_adj"] < qval_cutoff) & (data["mean_expression"] > mean_expr_cutoff)]
        sig_genes_top5 = sig_genes.nlargest(5, "logfoldchanges")
        texts = []
        for _, row in sig_genes_top5.iterrows():
            if row["logfoldchanges"] > 0:
                t = ax.text(
                    row["logfoldchanges"],
                    row["mean_expression"],
                    row["names"],
                    ha="right",
                    va="top",
                    color="r",
                )
                texts.append(t)
        # adjust_text(texts, ax=ax)
        ax.set_title(group)
        ax.set_xlabel("Log2FC")
        ax.set_ylabel("mean_expression")

    plt.suptitle("Volcano plot", fontsize=16)
    plt.tight_layout(rect=[0, 0, 1, 0.95])
    plt.savefig(save)
    plt.show()
    return DEG_dict



import numpy as np
import pandas as pd
import scanpy as sc
from anndata import AnnData
from sklearn.cluster import BisectingKMeans


def merge_big_cell(
    adata: AnnData,
    resolution: str,
    prefix: str,
    merge_tags: list[str],
    n: int = 30,
) -> AnnData:
    """Merge STomics cellbin data to metacell by axis and celltype.

    Args:
        adata (AnnData): adata object
        resolution (str): celltype tag in adata.obs
        prefix (str): prefix of output file
        merge_tags (list[str]): tags to merge
        n (int, optional): cell numbers to merge, defaults to 30

    Returns
    -------
        AnnData: merged adata object with resulution and merge_tags
    """
    adata_list = []
    for category in adata.obs[resolution].unique():
        sub_adata = adata[adata.obs[resolution] == category].copy()
        adata_list.append(sub_adata)
    merged_adatas = []
    merged_cluster_labels = pd.DataFrame(columns=["cell_id", "merged_cluster"])
    for i, st_adata in enumerate(adata_list):
        t = st_adata.shape[0] // n
        X = st_adata.obs[["x", "y"]].values
        spectral_clustering = BisectingKMeans(n_clusters=t, bisecting_strategy="largest_cluster", random_state=0)
        cluster_labels = spectral_clustering.fit_predict(X)
        # cluster_sizes = np.bincount(cluster_labels)
        # sorted_clusters = np.argsort(cluster_sizes)[::-1]
        # max_cluster_index = sorted_clusters[0]
        # while max_cluster_index
        cluster = st_adata.obs[resolution].iloc[0]
        adata_idx = adata.obs[resolution].unique().to_list().index(cluster)
        merged_cluster_labels_i = pd.DataFrame(
            list(zip(st_adata.obs_names, [f"{i-1}_{adata_idx}" for i in cluster_labels])),
            columns=["cell_id", "merged_cluster"],
        )
        merged_cluster_labels = pd.concat([merged_cluster_labels, merged_cluster_labels_i], axis=0)
        # 计算每个簇的中心坐标和gene count
        cluster_centers = []
        cluster_gene_counts = []
        cluster_cell_counts = []
        for cluster_id in np.unique(cluster_labels):
            cluster_spots = st_adata.obs.index[cluster_labels == cluster_id]
            cluster_center = np.mean(st_adata.obs.loc[cluster_spots, ["x", "y"]], axis=0)
            cluster_gene_count = np.sum(st_adata[cluster_spots].X, axis=0)
            cluster_centers.append(cluster_center)
            cluster_gene_counts.append(cluster_gene_count)
            cluster_cell_counts.append(cluster_spots.shape[0])
        # 构建合并后的adata对象
        adata_cluster = sc.AnnData(
            X=np.array(cluster_gene_counts)[:, 0, :],
            obsm={"spatial": np.array(cluster_centers)},
            obs=pd.DataFrame(
                cluster_cell_counts, index=np.arange(len(cluster_centers)), columns=["cluster_cell_counts"]
            ),
            var=adata.var,
        )
        adata_cluster.obs[resolution] = st_adata.obs[resolution].iloc[0]
        for tag in merge_tags:
            adata_cluster.obs[tag] = st_adata.obs[tag].iloc[0]
        merged_adatas.append(adata_cluster)
    merged_cluster_labels.to_csv(f"merged_cluster_labels_{prefix}.txt", sep="\t", index=None)
    # 合并所有合并后的adata对象
    merged_adata = sc.AnnData.concatenate(*merged_adatas, join="outer")
    return merged_adata



from typing import Literal, Optional, Union

import cv2
import matplotlib as mpl
import matplotlib.colors as mcolors
import matplotlib.gridspec as gridspec
import matplotlib.patches as patches
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
from anndata import AnnData
from matplotlib.axes import Axes
from matplotlib.offsetbox import AnchoredText
from matplotlib_scalebar.scalebar import ScaleBar

from ._utils import crop_to_square_with_padding, getDefaultColors, hex_to_rgb, int_to_rgb_idx


def cell_bin_plot(mask: str, res: pd.DataFrame, tag: str, colors: Optional[Union[list, dict[str, str]]]) -> np.ndarray:
    """Cell bin plot for stereo-seq data

    Args:
        mask (str): mask tif file path
        res (pd.DataFrame): obs dataframe of cell bin, three columns: x, y, tag
        tag (str): adata.obs.columns to plot
        colors (Sequence): color list corresponding to tag

    Returns
    -------
        np.ndarray: colored cellbin image array
    """
    # 读取二进制掩码图像
    image = cv2.imread(mask, cv2.IMREAD_GRAYSCALE)

    # 设置4邻域连接
    connectivity = 4
    # 使用connectedComponentsWithStats进行联通区域分析
    output = cv2.connectedComponentsWithStats(image, connectivity=connectivity)

    # 获取聚类标签和颜色
    clusters = res[tag].unique()
    # colors = getDefaultColors(cluster_number, type=colors)

    # 为每个聚类标签添加颜色列
    if isinstance(colors, list):
        res["color"] = res[tag].apply(lambda x: colors[clusters.tolist().index(x)])
    elif isinstance(colors, dict):
        res["color"] = res[tag].apply(lambda x: colors[x])

    # 将'x'和'y'从float类型转换为int类型
    res["x"] = res["x"].astype(int)
    res["y"] = res["y"].astype(int)

    # 将每个像素的联通区域标签添加到res中
    res["connectedComponentsLabel"] = output[1][res["y"].values, res["x"].values]

    # 创建一个颜色映射，将每个联通区域标签映射到对应的颜色
    colorMap = {
        connectedComponentsLabel: hex_to_rgb(color)
        for connectedComponentsLabel, color in zip(res["connectedComponentsLabel"].values, res["color"].values)
    }
    # 将背景（标签为0）的颜色设为黑色
    colorMap[0] = (0, 0, 0)

    # 将联通区域标签映射为颜色的索引
    res_idx = int_to_rgb_idx(output[1], colorMap)

    # 将索引映射为RGB图像
    image = np.uint8(res_idx)

    return image


def plot_cellbin_gradient(
    adata: AnnData,
    mask: str,
    tag: str,
    prefix: str,
    colors: str = "RdYlBu_r",
    dpi: float = 600,
    edge_cut: float = 300,
    add_scale_bar: bool = True,
    add_legend=True,
    background: Literal["white", "black"] = "white",
    scale: float = 0.5,
    length_fraction: float = 0.25,
    ax: Optional[Axes] = None,
    save: bool = True,
) -> Axes:
    """Plot cellbin gradient image

    Args:
        adata (AnnData): AnnData object
        mask (str): mask file path
        tag (str): tag name in adata.obs
        prefix (str): prefix of output file
        colors (str, optional): color palette, defaults to "RdYlBu_r"
        dpi (float, optional): dpi of output file, defaults to 600
        edge_cut (float, optional): pixels to retain in edge cutting,
            defaults to 300
        add_scale_bar (bool, optional): add scale bar or not, defaults
            to True
        background (Literal[&quot;white&quot;, &quot;black&quot;], optional):
            background color, defaults to "white"
        scale (float, optional): scale bar length, defaults to 0.5
        length_fraction (float, optional): scale bar length fraction,
            defaults to 0.25
        ax (Optional[Axes], optional): matplotlib axes, defaults to None
        save (bool, optional): save figure or not, defaults to True

    Returns
    -------
        Axes: matplotlib axes
    """
    res = pd.DataFrame(adata.obs, columns=["x", "y", tag], index=adata.obs.index)
    res = res.sort_values(by=tag)
    vmax = res[tag].quantile(0.95)  # top 5% largest value as vmax
    res[tag][res[tag] > vmax] = vmax
    vmin = res[tag].quantile(0.05)  # bottom 5% smallest value as vmin
    res[tag][res[tag] < vmin] = vmin

    clusters = res[tag].unique()
    # cluster_number = clusters.shape[0]
    cmap = plt.cm.get_cmap("RdYlBu_r")  # get color plette
    norm = mcolors.Normalize(vmin=clusters.min(), vmax=clusters.max())
    colors = {value: mcolors.rgb2hex(cmap(norm(value))) for value in clusters}  # get color for each cluster

    # plot cellbin gradient image
    im = cell_bin_plot(mask=mask, res=res, tag=tag, colors=colors)

    # cut black edge of im
    # non_black_coords = np.argwhere(im.sum(axis=2) > 0)
    # y1, x1 = non_black_coords.min(axis=0)
    # y2, x2 = non_black_coords.max(axis=0)
    # im = im[y1 - edge_cut : y2 + edge_cut, x1 - edge_cut : x2 + edge_cut]
    im = crop_to_square_with_padding(im, edge_cut=edge_cut)

    # replace black background with white
    if background == "white":
        text_color = "black"
        black_pixels = (im[:, :, 0] == 0) & (im[:, :, 1] == 0) & (im[:, :, 2] == 0)
        im[black_pixels] = [255, 255, 255]
    else:
        text_color = "white"

    # draw colorbar
    # if ax is None:
    #     fig, ax = plt.subplots(figsize=(5, 5), dpi=dpi, gridspec_kw={"wspace": 0, "hspace": 0})
    #     plt.subplots_adjust(0, 0, 1, 1)
    if save:
        fig, ax = plt.subplots(figsize=(5, 5), dpi=dpi, gridspec_kw={"wspace": 0, "hspace": 0})
        fig.patch.set_facecolor(background)
        plt.subplots_adjust(0, 0, 1, 1)
    ax.axis("off")
    ax.imshow(im)

    # add scale bar
    if add_scale_bar:
        scalebar = ScaleBar(
            scale,
            "um",
            length_fraction=length_fraction,
            frameon=False,
            color=text_color,
            location="lower right",
        )
        ax.add_artist(scalebar)

    # Create colorbar and add to ax
    if add_legend:
        cb = plt.colorbar(plt.cm.ScalarMappable(norm=norm, cmap=cmap), ax=ax, shrink=0.6)
        cb.solids.set_edgecolor("face")
        cb.solids.set_facecolor(background)

        # 设置colorbar的字体颜色为白色
        cb.ax.yaxis.set_tick_params(color=text_color)
        cb.ax.xaxis.set_tick_params(color=text_color)
        cb.ax.tick_params(axis="x", colors=text_color, labelsize=14)
        cb.ax.tick_params(axis="y", colors=text_color, labelsize=14)
    if save:
        outfig = f"{tag}_{prefix}_spatial.pdf"
        plt.savefig(outfig, dpi=dpi, format="pdf", bbox_inches="tight")
    return im


def plot_cellbin_discrete(
    adata: AnnData,
    mask: str,
    tag: str,
    prefix: str,
    colors: Optional[Union[int, dict[str, str], str]] = 9,
    dpi: int = 600,
    edge_cut: int = 300,
    background: Literal["white", "black"] = "white",
    add_scale_bar: bool = True,
    scale: float = 0.5,
    length_fraction: float = 0.25,
    ax: Optional[Axes] = None,
    add_legend=True,
    save: bool = True,
) -> Axes:
    """Plot discrete cellbin images.

    Args:
        adata (AnnData): AnnData object
        mask (str): mask file path
        tag (str): tag name in adata.obs
        prefix (str): prefix of output file
        colors (Literal, optional): color palette in getDefaultcolors,
            defaults to 9
        dpi (int, optional): dpi of output file, defaults to 600
        edge_cut (int, optional): pixels to retain in edge cutting,
            defaults to 300
        background (Literal[&quot;white&quot;, &quot;black&quot;], optional):
            background color, white or black, defaults to "white"
        add_scale_bar (bool, optional): add scale bar or not, defaults
            to True
        scale (float, optional): scale bar length, defaults to 0.5
        length_fraction (float, optional): scale bar length fraction,
            defaults to 0.25
        ax (Optional[Axes], optional): matplotlib axes, defaults to None
        add_legend (bool, optional): add legend or not, defaults to True
        save (bool, optional): save figure or not, defaults to True

    Returns
    -------
        Axes: matplotlib axes
    """
    res = pd.DataFrame(adata.obs, columns=["x", "y", tag], index=adata.obs.index)
    res = res.sort_values(by=tag)
    # res.to_csv(f"bin1clu_{tag}_{prefix}.txt", sep="\t", index=False)  # write to file
    clusters = res[tag].unique()
    cluster_number = clusters.shape[0]
    if isinstance(colors, int) or isinstance(colors, str):
        colors = getDefaultColors(cluster_number, type=colors)
        im = cell_bin_plot(mask=mask, res=res, tag=tag, colors=colors)
    elif isinstance(colors, dict):
        im = cell_bin_plot(mask=mask, res=res, tag=tag, colors=colors)
    else:
        raise TypeError("Invalid type for 'colors'. Must be an int, str or dict.")

    # cut black edge of im
    # non_black_coords = np.argwhere(im.sum(axis=2) > 0)
    # y1, x1 = non_black_coords.min(axis=0)
    # y2, x2 = non_black_coords.max(axis=0)
    # im = im[y1 - edge_cut : y2 + edge_cut, x1 - edge_cut : x2 + edge_cut]
    im = crop_to_square_with_padding(im, edge_cut=edge_cut)

    # replace black background with white
    if background == "white":
        text_color = "black"
        black_pixels = (im[:, :, 0] == 0) & (im[:, :, 1] == 0) & (im[:, :, 2] == 0)
        im[black_pixels] = [255, 255, 255]
    else:
        text_color = "white"

    # new fig for save
    if save:
        fig, ax = plt.subplots(figsize=(5, 5), dpi=dpi, gridspec_kw={"wspace": 0, "hspace": 0})
        fig.patch.set_facecolor(background)
        plt.subplots_adjust(0, 0, 1, 1)

    ax.axis("off")
    ax.imshow(im)

    # add scale bar
    if add_scale_bar:
        scalebar = ScaleBar(
            scale,
            "um",
            length_fraction=length_fraction,
            frameon=False,
            color=text_color,
            location="lower right",
        )
        ax.add_artist(scalebar)

    # Create a legend for the discrete colors
    if add_legend:
        legend_labels = [str(cluster) for cluster in clusters]  # Assuming clusters is a list of labels
        legend_patches = [mpl.patches.Patch(color=colors[i], label=legend_labels[i]) for i in range(cluster_number)]
        legend = ax.legend(handles=legend_patches, loc="lower left", fontsize=8, bbox_to_anchor=(1.05, 0))
        legend.set_title(tag, prop={"size": 8, "weight": "bold"})
        # set legend text color white
        legend.get_title().set_color(text_color)
        for text in legend.get_texts():
            text.set_color(text_color)

    # save figure
    if save:
        outfig = f"{tag}_{prefix}_spatial.pdf"
        plt.savefig(outfig, dpi=dpi, format="pdf", bbox_inches="tight")

    return im


def plot_zoom_cellbin(
    adata, mask, tag, prefix, zoom_position1, zoom_position2, colors=10, zoom_size=3000, height=5, edge_cut=300
):
    im = cv2.imread(mask, cv2.IMREAD_GRAYSCALE)
    im = crop_to_square_with_padding(im, edge_cut=edge_cut)
    figsize = im.shape
    print(figsize)
    zoom_size = 3000
    # 定义放大区域的坐标 (x, y, width, height)
    region1 = zoom_position1 + (zoom_size, zoom_size)
    region2 = zoom_position2 + (zoom_size, zoom_size)
    # 计算图片长宽
    width_ratios = figsize[1] / (figsize[0] / (zoom_size * 2) * zoom_size)
    print(width_ratios)
    width = (figsize[1] / (figsize[0] / height)) + ((figsize[1] / (figsize[0] / height)) / width_ratios)
    print(width)

    # 创建一个图形和网格
    fig = plt.figure(figsize=(width, height), facecolor="black")
    gs = gridspec.GridSpec(2, 2, width_ratios=[width_ratios, 1], hspace=0, wspace=0)

    # 绘制原图
    ax1 = plt.subplot(gs[:, 0])
    image = plot_cellbin_discrete(
        adata,
        mask=mask,
        tag=tag,
        prefix=prefix,
        colors=colors,
        dpi=1000,
        background="black",
        add_scale_bar=True,
        ax=ax1,
        add_legend=False,
        save=False,
    )
    ax1.axis("off")

    # 提取放大区域
    zoomed_region1 = image[region1[1] : region1[1] + region1[3], region1[0] : region1[0] + region1[2]]
    zoomed_region2 = image[region2[1] : region2[1] + region2[3], region2[0] : region2[0] + region2[2]]

    # 添加边框
    rect = patches.Rectangle((0, 0), figsize[1], figsize[0], linewidth=2, edgecolor="white", facecolor="none")
    ax1.add_patch(rect)

    # 在原图上标注放大区域（白色框体）
    rect1 = patches.Rectangle(
        (region1[0], region1[1]), region1[2], region1[3], linewidth=2, edgecolor="white", facecolor="none"
    )
    rect2 = patches.Rectangle(
        (region2[0], region2[1]), region2[2], region2[3], linewidth=2, edgecolor="white", facecolor="none"
    )
    ax1.add_patch(rect1)
    ax1.add_patch(rect2)

    # 添加标题
    text_box = AnchoredText(
        prefix, frameon=True, loc=2, pad=0.4, borderpad=0.1, prop={"backgroundcolor": "white", "fontsize": 20}
    )
    ax1.add_artist(text_box)

    # 在原图上添加序号
    ax1.text(
        region1[0] + 1500,
        region1[1] + 1500,
        "1",
        color="white",
        fontsize=25,
        fontweight="bold",
        ha="center",
        va="center",
    )
    ax1.text(
        region2[0] + 1500,
        region2[1] + 1500,
        "2",
        color="white",
        fontsize=25,
        fontweight="bold",
        ha="center",
        va="center",
    )

    # 绘制放大的区域1
    ax2 = plt.subplot(gs[0, 1])
    ax2.imshow(zoomed_region1)
    ax2.axis("off")

    # 添加边框
    rect = patches.Rectangle((0, 0), zoom_size, zoom_size, linewidth=2, edgecolor="white", facecolor="none")
    ax2.add_patch(rect)

    # 在放大区域1上添加标题
    text_box = AnchoredText(
        "1", frameon=True, loc=2, pad=0.4, borderpad=0.1, prop={"backgroundcolor": "white", "fontsize": 20}
    )
    ax2.add_artist(text_box)

    # 绘制放大的区域2
    ax3 = plt.subplot(gs[1, 1])
    ax3.imshow(zoomed_region2)
    ax3.axis("off")

    # 添加边框
    rect = patches.Rectangle((0, 0), zoom_size, zoom_size, linewidth=2, edgecolor="white", facecolor="none")
    ax3.add_patch(rect)

    # 在放大区域2上添加标题
    text_box = AnchoredText(
        "2", frameon=True, loc=2, pad=0.4, borderpad=0.1, prop={"backgroundcolor": "white", "fontsize": 20}
    )
    ax3.add_artist(text_box)
    # 调整布局，使图像拼接成一个矩形
    plt.subplots_adjust(left=0, right=1, top=1, bottom=0)
    fig.savefig(f"cellbin_plot_zoom_{prefix}.pdf", dpi=300, bbox_inches="tight")



"""Plotting for nhood heatmap."""
from typing import Optional

import matplotlib.pyplot as plt
import numpy as np
import seaborn as sns
from anndata import AnnData
from matplotlib.axes import Axes

from STutils.tl import nhood_enrichment


def nhood_heatmap(
    adata: AnnData,
    coord_type: str = "generic",
    library_key: str = "batch",
    radius: float = 30,
    cluster_key: str = "region",
    ax: Optional[Axes] = None,
    figsize: tuple = (6, 5),
    cmap: str = "YlGn",
    save: bool = True,
) -> Axes:
    """Plot neighborhood heatmap.

    Args:
        adata (AnnData): anndata object
        coord_type (str, optional): Type of coordinate system, defaults
            to "generic"
        library_key (str, optional): batch info, defaults to "batch"
        radius (float, optional): Compute the graph based on
            neighborhood radius, defaults to 30
        cluster_key (str, optional): region or cell cluster key,
            defaults to "region"
        ax (Optional[Axes], optional): mpl axes, defaults to None
        figsize (tuple, optional): fig size, defaults to (6, 5)
        cmap (str, optional): colormap, defaults to "YlGn"
        save (bool, optional): save or not, defaults to True

    Returns
    -------
        Axes: mpl axes
    """
    nhood_percents = nhood_enrichment(
        adata, coord_type=coord_type, library_key=library_key, radius=radius, cluster_key=cluster_key
    )
    # Remove the numbers on the diagonal
    nhood_percents = nhood_percents.mask(np.eye(len(nhood_percents), dtype=bool))
    # the numbers on the diagonal equal to max value in the matrix
    nhood_percents = nhood_percents.mask(np.eye(len(nhood_percents), dtype=bool), nhood_percents.max().max())
    # zscore nhood_percents
    # nhood_percents = (nhood_percents - nhood_percents.mean()) / nhood_percents.std()
    if ax is None:
        fig, ax = plt.subplots(figsize=figsize)
    sns.heatmap(nhood_percents, cmap=cmap, linewidth=0.3, linecolor="#929292", ax=ax)
    # set the NA value as grey color
    ax.set_facecolor("#dcdcdc")
    if save:
        outfig = f"{cluster_key}_nhood_heatmap.pdf"
        plt.savefig(outfig, dpi=300, format="pdf", bbox_inches="tight")
    return ax
